#!/usr/bin/env python
# -*- coding: utf-8 -*-

# py_tools_ds - A collection of geospatial data analysis tools that simplify standard
# operations when handling geospatial raster and vector data as well as projections.
#
# Copyright (C) 2016-2021
# - Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
# - Helmholtz Centre Potsdam - GFZ German Research Centre for Geosciences Potsdam,
#   Germany (https://www.gfz-potsdam.de/)
#
# This software was developed within the context of the GeoMultiSens project funded
# by the German Federal Ministry of Education and Research
# (project grant code: 01 IS 14 010 A-C).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_conversion
---------------

Tests for `py_tools_ds.geo.raster.conversion` module.
"""

import os
from unittest import TestCase

from osgeo import gdal
from shapely.geometry import MultiPolygon
import numpy as np

from py_tools_ds.geo.raster.conversion import raster2polygon
import py_tools_ds

tests_path = os.path.abspath(os.path.join(py_tools_ds.__file__, "..", "..", "tests"))


class Test_raster2polygon(TestCase):
    def setUp(self) -> None:
        try:
            # load a large mask containing an irregular binary shape with a complex footprint (27408x17604)
            ds_l = gdal.Open(os.path.join(tests_path, 'data', 'mask_large.tif'))
            self.mask_l = ds_l.ReadAsArray().astype(bool)
            self.gt_l = ds_l.GetGeoTransform()
            self.prj_l = ds_l.GetProjection()

        finally:
            del ds_l

        try:
            # load a large mask containing 239 features
            ds_mf = gdal.Open(os.path.join(tests_path, 'data', 'mask_many_features.tif'))
            self.mask_mf = ds_mf.ReadAsArray().astype(bool)
            self.gt_mf = ds_mf.GetGeoTransform()
            self.prj_mf = ds_mf.GetProjection()

        finally:
            del ds_mf

        self.testarr = np.array([[2, 0, 2, 0, 0],
                                 [0, 2, 0, 1, 2],
                                 [1, 2, 0, 1, 0],
                                 [1, 2, 2, 2, 1],
                                 [1, 1, 2, 1, 1]])

    def test_mask_large(self):
        poly = raster2polygon(self.mask_l, self.gt_l, self.prj_l, exact=False)
        self.assertIsInstance(poly, MultiPolygon)

    def test_mask_many_features(self):
        poly = raster2polygon(self.mask_mf, self.gt_mf, self.prj_mf, exact=False)
        self.assertIsInstance(poly, MultiPolygon)
        # self.assertEqual(len(poly.geoms), 239)
        self.assertEqual(len(poly.geoms), 234)

    def test_min_npx(self):
        poly = raster2polygon(self.mask_mf, self.gt_mf, self.prj_mf, exact=False, min_npx=5)
        self.assertIsInstance(poly, MultiPolygon)
        self.assertLess(len(poly.geoms), 239)

    def test_handle_multipart_polygons(self):
        # this causes gdal.Polygonize to produce some MultiPolygons instead pf Polygons which has to be handled
        poly = raster2polygon(~self.mask_mf.astype(bool), self.gt_mf, self.prj_mf, exact=True)
        self.assertIsInstance(poly, MultiPolygon)

    def test_DN2extract(self):
        for DN, nexp in zip(range(3), [5, 3, 4]):
            with self.subTest(DN=DN):
                poly = raster2polygon(self.testarr, self.gt_mf, self.prj_mf, exact=False, DN2extract=DN)
                self.assertEqual(len(poly.geoms), nexp)
