#!/usr/bin/env python
# -*- coding: utf-8 -*-

# py_tools_ds - A collection of geospatial data analysis tools that simplify standard
# operations when handling geospatial raster and vector data as well as projections.
#
# Copyright (C) 2016-2021
# - Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
# - Helmholtz Centre Potsdam - GFZ German Research Centre for Geosciences Potsdam,
#   Germany (https://www.gfz-potsdam.de/)
#
# This software was developed within the context of the GeoMultiSens project funded
# by the German Federal Ministry of Education and Research
# (project grant code: 01 IS 14 010 A-C).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_vector
----------_

Tests for `py_tools_ds.numeric.vector` module.
"""

from unittest import TestCase, main

from py_tools_ds.numeric.vector import find_nearest


class Test_find_nearest(TestCase):
    def setUp(self):
        self.array = [330000.1,
                      330010.19999999995,
                      330020.29999999993,
                      330030.3999999999,
                      330040.4999999999,
                      330050.59999999986,
                      330060.69999999984,
                      330070.7999999998,
                      330080.8999999998,
                      330090.99999999977
                      ]
        self.value = 335442.984382

    def test_round_alg_off(self):
        out = find_nearest(self.array, 330068, roundAlg='off')
        self.assertEqual(out, 330060.69999999984)

    def test_round_alg_on(self):
        out = find_nearest(self.array, 330068, roundAlg='on')
        self.assertEqual(out, 330070.7999999998)

    def test_round_alg_auto(self):
        out = find_nearest(self.array, 330068, roundAlg='auto')
        self.assertEqual(out, 330070.7999999998)

    def test_round_alg_exact_hit(self):
        for roundAlg in ['off', 'on', 'auto']:
            out = find_nearest(self.array, 330060.69999999984, roundAlg=roundAlg)
            self.assertEqual(out, 330060.69999999984)

    def test_extrapolate(self):
        out = find_nearest(self.array, 330110, extrapolate=True, roundAlg='auto')
        self.assertEqual(out, 330111.19999999972)

        out = find_nearest(self.array, 329980, extrapolate=True, roundAlg='auto')
        self.assertEqual(out, 329979.90000000002)

    def test_val_outside(self):
        with self.assertRaises(ValueError):
            find_nearest(self.array, 330110, roundAlg='auto')

    def test_exclude_val(self):
        out = find_nearest(self.array, 330060.69999999984, roundAlg='auto', exclude_val=True)
        self.assertNotEqual(out, 330060.69999999984)

    def test_tolerance(self):
        # value below
        out = find_nearest(self.array, 330060.6999999, roundAlg='off', tolerance=0.0001)
        self.assertEqual(out, 330060.69999999984)  # would return 330050.59999999986 without tolerance

        # value above
        out = find_nearest(self.array, 330060.7, roundAlg='on', tolerance=0.0001)
        self.assertEqual(out, 330060.69999999984)  # would return 330070.7999999998 without tolerance


if __name__ == '__main__':
    main()
